package main

import (
	"flag"
	"fmt"
	"os"
	"os/signal"
	"syscall"

	"git.ctrlz.es/mgdelacroix/birthdaybot/model"
	"git.ctrlz.es/mgdelacroix/birthdaybot/server"
	"github.com/charmbracelet/log"
)

func loggerFromConfig(config *model.Config) *log.Logger {
	// ToDo: apply config to logger
	return log.New(os.Stderr)
}

func main() {
	configFlag := flag.String("config", "birthdaybot.yml", "path to the configuration file")
	flag.Parse()

	config, err := model.ReadConfig(*configFlag)
	if err != nil {
		fmt.Fprintf(os.Stderr, "ERROR: error reading config %q: %s\n", *configFlag, err)
		os.Exit(1)
	}

	if err := config.IsValid(); err != nil {
		fmt.Fprintf(os.Stderr, "ERROR: invalid config %q: %s\n", *configFlag, err)
		os.Exit(1)
	}

	srv, err := server.New(
		server.WithLogger(loggerFromConfig(config)),
		server.WithConfig(config),
	)
	if err != nil {
		fmt.Fprintf(os.Stderr, "ERROR: error creating server: %s\n", err)
		os.Exit(1)
	}

	c := make(chan os.Signal, 1)
	signal.Notify(c, syscall.SIGINT, syscall.SIGTERM)

	srv.Start()
	s := <-c
	srv.Logger.Info("received signal, stopping", "signal", s)
	srv.Stop()
}
